"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const commonHelpers = require("../common/helpers");
class SolutionIdCommandParameter {
    constructor($remoteProjectService) {
        this.$remoteProjectService = $remoteProjectService;
        this.mandatory = false;
    }
    validate(validationValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (validationValue) {
                let app = yield this.$remoteProjectService.getSolutionData(validationValue);
                return !!app;
            }
            return false;
        });
    }
}
class CloudListProjectsCommand {
    constructor($logger, $remoteProjectService, $prompter, $options) {
        this.$logger = $logger;
        this.$remoteProjectService = $remoteProjectService;
        this.$prompter = $prompter;
        this.$options = $options;
        this.allowedParameters = [new SolutionIdCommandParameter(this.$remoteProjectService)];
    }
    printList(names, appName) {
        let isProject = !!appName;
        let headers = ["#", `${isProject ? 'Project' : 'App'} name`];
        let data = names.map((name, index) => [(++index).toString(), name]);
        let table = commonHelpers.createTable(headers, data);
        if (isProject) {
            this.$logger.out(`Projects for ${appName} app:`);
        }
        this.$logger.out(table.toString());
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let apps = yield this.$remoteProjectService.getAvailableAppsAndSolutions();
            let slnName = args[0];
            if (!slnName) {
                let appDisplayNames = apps.map(app => app.colorizedDisplayName);
                if (this.$options.all || !commonHelpers.isInteractive() || appDisplayNames.length === 1) {
                    this.printList(appDisplayNames);
                    return;
                }
                else {
                    slnName = yield this.$prompter.promptForChoice("Select solution for which to list projects:", appDisplayNames);
                }
            }
            let projects = (yield this.$remoteProjectService.getProjectsForSolution(slnName)).map(proj => proj.Name);
            this.printList(projects, slnName);
        });
    }
}
exports.CloudListProjectsCommand = CloudListProjectsCommand;
$injector.registerCommand("cloud|*list", CloudListProjectsCommand);
class CloudExportProjectsCommand {
    constructor($errors, $remoteProjectService, $prompter, $project) {
        this.$errors = $errors;
        this.$remoteProjectService = $remoteProjectService;
        this.$prompter = $prompter;
        this.$project = $project;
        this.allowedParameters = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let projectIdentifier = args[1];
            let slnName = args[0];
            if (!slnName) {
                let all = (yield this.$remoteProjectService.getAvailableAppsAndSolutions()).map(sln => sln.colorizedDisplayName) || [];
                slnName = yield this.$prompter.promptForChoice("Select solution to export", all);
                let projects = (yield this.$remoteProjectService.getProjectsForSolution(slnName)).map(proj => proj.Name);
                let exportSolutionItem = "Export the whole solution";
                projects.push(exportSolutionItem);
                let selection = yield this.$prompter.promptForChoice("Select project to export", projects);
                if (selection !== exportSolutionItem) {
                    projectIdentifier = selection;
                }
            }
            if (projectIdentifier) {
                let projectName = yield this.$remoteProjectService.getProjectName(slnName, projectIdentifier);
                yield this.$remoteProjectService.exportProject(slnName, projectName);
            }
            else {
                yield this.$remoteProjectService.exportSolution(slnName);
            }
        });
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let solutionNames = (yield this.$remoteProjectService.getAvailableAppsAndSolutions()).map(sln => sln.colorizedDisplayName);
            if (!solutionNames || !solutionNames.length) {
                this.$errors.failWithoutHelp("You do not have any projects in the cloud.");
            }
            if (this.$project.projectData) {
                this.$errors.failWithoutHelp("Cannot create project in this location because the specified directory is part of an existing project. Switch to or specify another location and try again.");
            }
            if (args && args.length) {
                if (args.length > 2) {
                    this.$errors.fail("This command accepts maximum two parameters - solution name and project name.");
                }
                let slnName = args[0];
                if (args[1]) {
                    yield this.$remoteProjectService.getProjectName(slnName, args[1]);
                }
                else {
                    let projectNames = (yield this.$remoteProjectService.getProjectsForSolution(slnName)).map(sln => sln.Name);
                    if (!projectNames.length) {
                        this.$errors.failWithoutHelp(`Solution ${slnName} does not have any projects.`);
                    }
                }
            }
            else if (!commonHelpers.isInteractive()) {
                this.$errors.fail("When console is not interactive, you have to provide at least one argument.");
            }
            return true;
        });
    }
}
exports.CloudExportProjectsCommand = CloudExportProjectsCommand;
$injector.registerCommand("cloud|export", CloudExportProjectsCommand);
